#-*= coding: utf-8 -*-
""" Execute Hebrew Python scripts """

__author__ = 'Nir Soffer <nirs@freeshell.org'
__credits__ = 'Noam Raphael, Beni Cherniavksy'
__version__ = '0.1'

import codecs
import token
import cStringIO

from hpy import htokenize, hebrew

DEBUG = 0

def mangle(s):
    """ Return ASCII safe name for hpython identifier 
    e.g. פריט ==> hpy_d7a4d7a8d799d798
    """
    return 'hpy_' + s.encode('utf-8').encode('hex')

def translate(readline):
    """ Translate HPython source to Python source """
    result = cStringIO.StringIO()
    position = 0
    indent = ''
    
    for (type, string, (srow, scol), (erow, ecol), line) \
        in htokenize.generate_tokens(readline):
        
        # Add missing whitespace before tokens
        result.write(' ' * (scol - position))
        position = ecol
        
        if type == token.NAME:
            if string in hebrew.keywords:
                result.write(hebrew.keywords[string])
            else:
                result.write(mangle(string))
        elif type == token.INDENT:
            indent = string
            result.write(indent)
        elif type == token.DEDENT:
            indent = ' ' * ecol
            result.write(indent)
        elif type == token.NEWLINE:
            result.write(string + indent)
        else:
            result.write(string.encode('utf-8'))
    
    return result.getvalue()

def printTokens(readline):
    """ For debugging """
    for (type, string, (srow, scol), (erow, ecol), line) \
        in htokenize.generate_tokens(readline):
        print '%s (%d, %d): "%s"' % (token.tok_name[type], scol, ecol, string.encode('utf-8'))

def execute(path):
    """ Execute hpython source at path """
    if DEBUG: 
        printTokens(codecs.open(path, 'r', 'utf-8').readline)
    source = translate(codecs.open(path, 'r', 'utf-8').readline)
    if DEBUG: 
        print source
    code = compile(source, path, 'exec')
    sandbox = {}
    exec code in sandbox
